import requests
import json

import yaml

with open("./v.1.0.0.yaml", 'r') as stream:
    try:
        data = yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        print(exc)


class AppSyncClient:
    latest_version = '1.0.0'
    available_ds = [int(e) for e in data['available_ds'].split(", ")]
    available_ds_num = len(available_ds)

    def __init__(self, api_key, endpoint_url):
        self.api_key = api_key
        self.endpoint_url = endpoint_url
        self.headers = {
                'Content-Type': "application/graphql",
                'x-api-key': api_key,
                'cache-control': "no-cache"
        }

    def execute_gql(self, query):
        payload_obj = {"query": query}
        payload = json.dumps(payload_obj)
        response = requests.request("POST",
                                    self.endpoint_url,
                                    data=payload,
                                    headers=self.headers)
        return response

    def get_factual(self, ds: int, version: str = "1.0.0"):
        assert ds in self.available_ds, "Make sure choose one of ds_num"
        ds = "{}".format(str(ds))
        version = "\"{}\"".format(version)

        query = """
                query GetFactual{{
                    getFactual(
                        ds: {0}, 
                        version: {1}) {{
                               factual
                            }}
                        }}
                """.format(ds, version)

        res = self.execute_gql(query).json()
        return res['data']['getFactual']['factual']


if __name__ == "__main__":

    api_key = "da2-hgitmt7z45he3mlvrltgmwqlhm"
    endpoint_url = "https://3oedq5prgveqvdax2xtkc2lv34.appsync-api.us-east-1.amazonaws.com/graphql"

    client = AppSyncClient(api_key=api_key, endpoint_url=endpoint_url)
    print(client.available_ds)
    print(client.latest_version)
    print(client.available_ds_num)

    client.get_factual(ds=2)
    # ds = 1
    #
    # query = """
    #         query ListFactual {{
    #                   listFactual(
    #                     input: {{
    #                         ds: {0},
    #                         version: {1},
    #                         factual: {2}
    #                     }}
    #                  )
    #                   {{
    #                     ds
    #                     version
    #                     factual
    #                   }}
    #                 }}
    #         """.format(1, "\"1.0.4\"", "\"this-is-version4\"")
    # print(type(query))
    # print(query)
    # res = client.execute_gql(query).json()
    # print(res)



import json
import yaml


def read_data(json_path, stop_idx, cited):
    read_idx = 0
    ds_numbs = []
    art_numbs = []

    with open(json_path) as fin:
        for each_line in fin:
            data = json.loads(each_line)
            test_id = data['testid']
            features_content_gov = data['gov']
            features_content_art = data['art']
            label = int(data['label'][0])
            print(test_id, label)

            ds_numb = test_id.split('_')[0]
            art_numb = test_id.split('_')[1][1:-1]
            cited_art_numb = cited[art_numb].split(', ')

            # print(ds_numb, art_numb, label)
            # print(cited_art_numb)

            if label == 0:
                # print('DS{}'.format(ds_numb))
                assert 'DS{}'.format(ds_numb) not in cited_art_numb
            elif label == 1:
                print(ds_numb, art_numb, label)
                print(cited_art_numb)
                print('DS{}'.format(ds_numb))
                assert 'DS{}'.format(ds_numb) in cited_art_numb

            ds_numbs.append(ds_numb)
            art_numbs.append(art_numb)

            if read_idx == stop_idx:
                break
            else:
                read_idx += 1
    ds_numbs_set = set(ds_numbs)
    art_numbs_set = set(art_numbs)

    ds_numbs_set_list = []
    for e in ds_numbs_set:
        ds_numbs_set_list.append(int(e))
    ds_numbs_set_list.sort()

    art_numbs_set_list = []
    for e in art_numbs_set:
        art_numbs_set_list.append(e)
    art_numbs_set_list.sort()


if __name__ == "__main__":
    with open("./data/cited.yaml", 'r') as stream:
        try:
            cited = yaml.safe_load(stream)
            keys = list(cited.keys())
            keys.sort()

        except yaml.YAMLError as exc:
            print(exc)

    test_inst_num = 2287
    train_inst_num = 9153
    read_data("./data/test_data.json", 0, cited)

    # read_data("./data/test_data.json", test_inst_num-1, cited)
    # read_data("./data/train_data.json", train_inst_num-1, cited)

    pass



import yaml

from deepwto.graphql import AppSyncClient
from deepwto.Constants import *

if __name__ == "__main__":

    with open("./data/gatt.yaml", 'r') as stream:
        try:
            data = yaml.safe_load(stream)
            keys = list(data.keys())
            keys.sort()

        except yaml.YAMLError as exc:
            print(exc)

    client = AppSyncClient(api_key=api_key, endpoint_url=endpoint_url)

    for key in keys:
        print(key)
        article = "\"{}\"".format(key)
        version = "\"{}\"".format("1.0.0")
        content = "\"\"\"{}\"\"\"".format(data[key])

        query = """
                mutation CreateGATT {{
                          createGATT(
                            input: {{
                                article: {0},
                                version: {1},
                                content: {2}
                            }}
                         )
                          {{
                            article
                            version
                            content
                          }}
                        }}
                """.format(article, version, content)
        res = client.execute_gql(query).json()
        print(res)




# import sys
import pickle

from deepwto.graphql import AppSyncClient
from deepwto.Constants import *


def load_pkl(pickle_path):
    with open(pickle_path, 'rb') as f:
        py_obj = pickle.load(f)
    return py_obj


if __name__ == "__main__":
    path = './data/factual.pkl'

    classes = load_pkl(path)
    keys = list(classes.keys())
    keys.sort()
    print(keys)
    print(len(classes))

    client = AppSyncClient(api_key=api_key, endpoint_url=endpoint_url)

    over_array = []
    for key in keys:
        # Check size over 400 KB : but looks like it's okay to the longest
        # case of 480544. So we comment-out the size checking block
        # byte_size = sys.getsizeof(classes[key])
        # dynamo_limit = 400000
        # limit_over = byte_size > dynamo_limit
        # if limit_over:
        #     over_array.append(key)

        ds = key
        version = "\"{}\"".format("1.0.0")
        factual = "\"\"\"{}\"\"\"".format(classes[key])

        query = """
                mutation CreateFactual {{
                          createFactual(
                            input: {{
                                ds: {0},
                                version: {1},
                                factual: {2}
                            }}
                         )
                          {{
                            ds
                            version
                            factual
                          }}
                        }}
                """.format(ds, version, factual)
        res = client.execute_gql(query).json()



import json
import yaml

from deepwto.graphql import AppSyncClient
from deepwto.Constants import *


def read_data(json_path, stop_idx, split):
    read_idx = 0
    split = "\"{}\"".format(split)

    with open(json_path) as fin:
        for each_line in fin:
            data = json.loads(each_line)
            test_id = data['testid']
            features_content_gov = data['gov']
            features_content_art = data['art']
            label = int(data['label'][0])
            # print(test_id, label)

            ds_numb = test_id.split('_')[0]
            art_numb = test_id.split('_')[1][1:-1]
            ds_art = ds_numb + '_' + art_numb
            print(ds_art, label)

            if label == 0:
                cited = 'false'
            elif label == 1:
                cited = 'true'

            client = AppSyncClient(api_key=api_key, endpoint_url=endpoint_url)

            ds_art = "\"{}\"".format(ds_art)
            version = "\"{}\"".format("1.0.0")

            query = """
                    mutation CreateLabel {{
                              createLabel(
                                input: {{
                                    ds_art: {0},
                                    version: {1},
                                    cited: {2},
                                    split: {3}
                                }}
                             )
                              {{
                                ds_art
                                version
                                cited
                                split
                              }}
                            }}
                    """.format(ds_art, version, cited, split)
            res = client.execute_gql(query).json()
            print(res)
            assert 'errors' not in res.keys()

            if read_idx == stop_idx:
                break
            else:
                read_idx += 1


if __name__ == "__main__":
    test_inst_num = 2287
    train_inst_num = 9153
    # read_data("./data/test_data.json", 0)

    # read_data("./data/test_data.json", test_inst_num-1, 'test')
    read_data("./data/train_data.json", train_inst_num-1, 'train')

    pass

